@@ -2,19 +2,25 @@
 """
 https://www.w3.org/TR/shacl/#sparql-constraint-components
 """
-import re
+import typing
 
-from typing import Dict, List, Tuple, Union
+from typing import Any, Dict, List, Set, Tuple, Type, Union
 
 import rdflib
 
 from pyshacl.constraints.constraint_component import ConstraintComponent
 from pyshacl.constraints.sparql.sparql_based_constraints import SPARQLQueryHelper
-from pyshacl.consts import SH, RDF_type, SH_ask, SH_message, SH_parameter, SH_select
-from pyshacl.errors import ConstraintLoadError, ReportableRuntimeError, ValidationFailure
+from pyshacl.consts import SH, RDF_type, SH_ask, SH_message, SH_parameter, SH_path, SH_select
+from pyshacl.errors import ConstraintLoadError, ValidationFailure
+from pyshacl.parameter import SHACLParameter
 from pyshacl.pytypes import GraphLike
 
 
+if typing.TYPE_CHECKING:
+    from pyshacl.shape import Shape
+    from pyshacl.shapes_graph import ShapesGraph
+
+
 SH_nodeValidator = SH.term('nodeValidator')
 SH_propertyValidator = SH.term('propertyValidator')
 SH_validator = SH.term('validator')
@@ -24,20 +30,94 @@ SH_ConstraintComponent = SH.term('ConstraintComponent')
 SH_SPARQLSelectValidator = SH.term('SPARQLSelectValidator')
 SH_SPARQLAskValidator = SH.term('SPARQLAskValidator')
 
-invalid_parameter_names = {'this', 'shapesGraph', 'currentShape', 'path', 'PATH', 'value'}
+
+class BoundShapeValidatorComponent(ConstraintComponent):
+    def __init__(self, constraint, shape: 'Shape', validator):
+        """
+        Create a new custom constraint, by applying a ConstraintComponent and a Validator to a Shape
+        :param constraint: The source ConstraintComponent, this is needed to bind the parameters in the query_helper
+        :type constraint: SPARQLConstraintComponent
+        :param shape:
+        :type shape: Shape
+        :param validator:
+        :type validator: AskConstraintValidator | SelectConstraintValidator
+        """
+        super(BoundShapeValidatorComponent, self).__init__(shape)
+        self.constraint = constraint
+        self.validator = validator
+        params = constraint.parameters
+        self.query_helper = SPARQLQueryHelper(
+            self.shape, validator.node, validator.query_text, params, messages=validator.messages
+        )
+        # Setting self.shape into QueryHelper automatically applies query_helper.bind_params and bind_messages
+        self.query_helper.collect_prefixes()
+
+    @classmethod
+    def constraint_parameters(cls):
+        # TODO:coverage: this is never used for this constraint?
+        return [SH_validator, SH_nodeValidator, SH_propertyValidator]
+
+    @classmethod
+    def constraint_name(cls):
+        return "ConstraintComponent"
+
+    @classmethod
+    def shacl_constraint_class(cls):
+        # TODO:coverage: this is never used for this constraint?
+        return SH_ConstraintComponent
+
+    def evaluate(self, target_graph: GraphLike, focus_value_nodes: Dict, _evaluation_path: List):
+        """
+        :type focus_value_nodes: dict
+        :type target_graph: rdflib.Graph
+        """
+        reports = []
+        non_conformant = False
+        extra_messages = self.query_helper.messages or None
+        rept_kwargs = {
+            # TODO, determine if we need sourceConstraint here
+            #  'source_constraint': self.validator.node,
+            'constraint_component': self.constraint.node,
+            'extra_messages': extra_messages,
+        }
+        for f, value_nodes in focus_value_nodes.items():
+            # we don't use value_nodes in the sparql constraint
+            # All queries are done on the corresponding focus node.
+            try:
+                violations = self.validator.validate(f, value_nodes, target_graph, self.query_helper)
+            except ValidationFailure as e:
+                raise e
+            if not self.shape.is_property_shape:
+                result_val = f
+            else:
+                result_val = None
+            for v in violations:
+                non_conformant = True
+                if isinstance(v, bool) and v is True:
+                    # TODO:coverage: No test for when violation is `True`
+                    rept = self.make_v_result(target_graph, f, value_node=result_val, **rept_kwargs)
+                elif isinstance(v, tuple):
+                    t, p, v = v
+                    if v is None:
+                        v = result_val
+                    rept = self.make_v_result(target_graph, t or f, value_node=v, result_path=p, **rept_kwargs)
+                else:
+                    rept = self.make_v_result(target_graph, f, value_node=v, **rept_kwargs)
+                reports.append(rept)
+        return (not non_conformant), reports
 
 
 class SPARQLConstraintComponentValidator(object):
     validator_cache: Dict[Tuple[int, str], Union['SelectConstraintValidator', 'AskConstraintValidator']] = {}
 
-    def __new__(cls, shacl_graph, node, *args, **kwargs):
+    def __new__(cls, shacl_graph: 'ShapesGraph', node, *args, **kwargs):
         cache_key = (id(shacl_graph.graph), str(node))
         found_in_cache = cls.validator_cache.get(cache_key, False)
         if found_in_cache:
             return found_in_cache
         sg = shacl_graph.graph
         type_vals = set(sg.objects(node, RDF_type))
-        validator_type = None
+        validator_type: Union[Type[SelectConstraintValidator], Type[AskConstraintValidator], None] = None
         if len(type_vals) > 0:
             if SH_SPARQLSelectValidator in type_vals:
                 validator_type = SelectConstraintValidator
@@ -63,9 +143,9 @@ class SPARQLConstraintComponentValidator(object):
         cls.validator_cache[cache_key] = validator
         return validator
 
-    def apply_to_shape_via_constraint(self, constraint, shape, **kwargs):
+    def apply_to_shape_via_constraint(self, constraint, shape, **kwargs) -> BoundShapeValidatorComponent:
         """
-
+        Create a new Custom Constraint (BoundShapeValidatorComponent)
         :param constraint:
         :type constraint: SPARQLConstraintComponent
         :param shape:
@@ -87,32 +167,10 @@ class SPARQLConstraintComponentValidator(object):
                 "Validator for a NodeShape or a PropertyShape must be of type SPARQLSelectValidator.",
                 "https://www.w3.org/TR/shacl/#ConstraintComponent",
             )
-        bind_map = {}
-        for m in constraint.mandatory_parameters:
-            name = constraint.parameter_name(m)
-            if name in invalid_parameter_names:
-                # TODO:coverage: No test for this case
-                raise ReportableRuntimeError("Parameter name {} cannot be used.".format(name))
-            shape_params = set(shape.objects(m.path()))
-            if len(shape_params) < 1:
-                # TODO:coverage: No test for this case
-                raise ReportableRuntimeError("Shape does not have mandatory parameter {}.".format(str(m.path())))
-            # TODO: Can shapes have more than one value for the predicate?
-            # Just use one for now.
-            bind_map[name] = next(iter(shape_params))
-        for o in constraint.optional_parameters:
-            name = constraint.parameter_name(o)
-            if name in invalid_parameter_names:
-                # TODO:coverage: No test for this case
-                raise ReportableRuntimeError("Parameter name {} cannot be used.".format(name))
-            shape_params = set(shape.objects(o.path()))
-            if len(shape_params) > 0:
-                # TODO: Can shapes have more than one value for the predicate?
-                # Just use one for now.
-                bind_map[name] = next(iter(shape_params))
-        return BoundShapeValidatorComponent(constraint, shape, self, bind_map)
 
-    def __init__(self, shacl_graph, node, **kwargs):
+        return BoundShapeValidatorComponent(constraint, shape, self)
+
+    def __init__(self, shacl_graph: 'ShapesGraph', node, **kwargs):
         initialised = getattr(self, 'initialised', False)
         if initialised:
             return
@@ -132,10 +190,10 @@ class SPARQLConstraintComponentValidator(object):
 
 
 class AskConstraintValidator(SPARQLConstraintComponentValidator):
-    def __new__(cls, shacl_graph, node, *args, **kwargs):
+    def __new__(cls, shacl_graph: 'ShapesGraph', node, *args, **kwargs):
         return object.__new__(cls)
 
-    def __init__(self, shacl_graph, node, *args, **kwargs):
+    def __init__(self, shacl_graph: 'ShapesGraph', node, *args, **kwargs):
         super(AskConstraintValidator, self).__init__(shacl_graph, node, **kwargs)
         g = shacl_graph.graph
         ask_vals = set(g.objects(node, SH_ask))
@@ -154,7 +212,7 @@ class AskConstraintValidator(SPARQLConstraintComponentValidator):
             )
         self.query_text = ask_val.value
 
-    def validate(self, focus, value_nodes, target_graph, query_helper=None, bind_vals=None):
+    def validate(self, focus, value_nodes, target_graph, query_helper=None, new_bind_vals=None):
         """
 
         :param focus:
@@ -162,10 +220,13 @@ class AskConstraintValidator(SPARQLConstraintComponentValidator):
         :param query_helper:
         :param target_graph:
         :type target_graph: rdflib.Graph
-        :param bind_vals:
+        :param new_bind_vals:
         :return:
         """
-        bind_vals = bind_vals or {}
+        param_bind_vals = query_helper.param_bind_map if query_helper else {}
+        new_bind_vals = new_bind_vals or {}
+        bind_vals = param_bind_vals.copy()
+        bind_vals.update(new_bind_vals)
         violations = []
         for v in value_nodes:
             if query_helper is None:
@@ -190,10 +251,10 @@ class AskConstraintValidator(SPARQLConstraintComponentValidator):
 
 
 class SelectConstraintValidator(SPARQLConstraintComponentValidator):
-    def __new__(cls, shacl_graph, node, *args, **kwargs):
+    def __new__(cls, shacl_graph: 'ShapesGraph', node, *args, **kwargs):
         return object.__new__(cls)
 
-    def __init__(self, shacl_graph, node, *args, **kwargs):
+    def __init__(self, shacl_graph: 'ShapesGraph', node, *args, **kwargs):
         super(SelectConstraintValidator, self).__init__(shacl_graph, node, **kwargs)
         g = shacl_graph.graph
         select_vals = set(g.objects(node, SH_select))
@@ -212,7 +273,7 @@ class SelectConstraintValidator(SPARQLConstraintComponentValidator):
             )
         self.query_text = select_val.value
 
-    def validate(self, focus, value_nodes, target_graph, query_helper=None, bind_vals=None):
+    def validate(self, focus, value_nodes, target_graph, query_helper=None, new_bind_vals=None):
         """
 
         :param focus:
@@ -220,17 +281,22 @@ class SelectConstraintValidator(SPARQLConstraintComponentValidator):
         :param query_helper:
         :param target_graph:
         :type target_graph: rdflib.Graph
-        :param bind_vals:
+        :param new_bind_vals:
         :return:
         """
-        bind_vals = bind_vals or {}
+        param_bind_vals = query_helper.param_bind_map if query_helper else {}
+        new_bind_vals = new_bind_vals or {}
+        bind_vals = param_bind_vals.copy()
+        bind_vals.update(new_bind_vals)
         for v in value_nodes:
             if query_helper is None:
                 # TODO:coverage: No test for this case when query_helper is None
                 init_binds = {}
                 sparql_text = self.query_text
             else:
-                init_binds, sparql_text = query_helper.pre_bind_variables(focus, valuenode=v)
+                init_binds, sparql_text = query_helper.pre_bind_variables(
+                    focus, valuenode=v, extravars=bind_vals.keys()
+                )
                 sparql_text = query_helper.apply_prefixes(sparql_text)
                 init_binds.update(bind_vals)
             results = target_graph.query(sparql_text, initBindings=init_binds)
@@ -265,168 +331,138 @@ class SelectConstraintValidator(SPARQLConstraintComponentValidator):
             return violations
 
 
-class BoundShapeValidatorComponent(ConstraintComponent):
-    def __init__(self, constraint, shape, validator, param_bind_map):
-        """
+class CustomConstraintComponentFactory(object):
+    __slots__: Tuple = tuple()
 
-        :param constraint:
-        :type constraint: SPARQLConstraintComponent
-        :param shape:
-        :type shape: pyshacl.shape.Shape
-        :param validator:
-        :type validator: AskConstraintValidator | SelectConstraintValidator
-        :param param_bind_map:
-        :type param_bind_map: dict
-        """
-        super(BoundShapeValidatorComponent, self).__init__(shape)
-        self.constraint = constraint
-        self.validator = validator
-        self.param_bind_map = param_bind_map
-        self.query_helper = SPARQLQueryHelper(
-            self.shape, validator.node, validator.query_text, messages=validator.messages
-        )
-        self.query_helper.collect_prefixes()
-        self.message_bind_map = {}
-        message_var_finder = re.compile(r"([\s()\"\'])\{[\$\?](\w+)\}", flags=re.M)
-        var_replacers = {}
-        self.messages = set()
-        for m in self.query_helper.messages:
-            m_val = str(m.value)
-            finds = message_var_finder.findall(m_val)
-            if len(finds) < 1:
-                self.messages.add(m)
-                continue
-            for f in finds:
-                variable = f[1]
-                if variable not in param_bind_map.keys():
-                    continue
-                try:
-                    replacer = var_replacers[variable]
-                except KeyError:
-                    replacer = re.compile(r"([\s()\"\'])\{[\$\?]" + f[1] + r"\}", flags=re.M)
-                    var_replacers[variable] = replacer
-                m_val = replacer.sub(r"\g<1>{}".format(param_bind_map[variable].value), m_val, 1)
-            self.messages.add(rdflib.Literal(m_val, lang=m.language, datatype=m.datatype))
-
-    @classmethod
-    def constraint_parameters(cls):
-        # TODO:coverage: this is never used for this constraint?
-        return [SH_validator, SH_nodeValidator, SH_propertyValidator, SH_parameter]
-
-    @classmethod
-    def constraint_name(cls):
-        return "ConstraintComponent"
-
-    @classmethod
-    def shacl_constraint_class(cls):
-        # TODO:coverage: this is never used for this constraint?
-        return SH_ConstraintComponent
-
-    def evaluate(self, target_graph: GraphLike, focus_value_nodes: Dict, _evaluation_path: List):
-        """
-        :type focus_value_nodes: dict
-        :type target_graph: rdflib.Graph
-        """
-        reports = []
-        non_conformant = False
-        extra_messages = self.messages or None
-        rept_kwargs = {
-            # TODO, determine if we need sourceConstraint here
-            #  'source_constraint': self.validator.node,
-            'constraint_component': self.constraint.node,
-            'extra_messages': extra_messages,
-        }
-        for f, value_nodes in focus_value_nodes.items():
-            # we don't use value_nodes in the sparql constraint
-            # All queries are done on the corresponding focus node.
-            try:
-                violations = self.validator.validate(
-                    f, value_nodes, target_graph, self.query_helper, self.param_bind_map
+    def __new__(cls, shacl_graph: 'ShapesGraph', node):
+        self: List[Any] = list()
+        self.append(shacl_graph)
+        self.append(node)
+        optional_params = []
+        mandatory_params = []
+        param_nodes = set(shacl_graph.objects(node, SH_parameter))
+        if len(param_nodes) < 1:
+            # TODO:coverage: we don't have any tests for invalid constraints
+            raise ConstraintLoadError(
+                "A sh:ConstraintComponent must have at least one value for sh:parameter",
+                "https://www.w3.org/TR/shacl/#constraint-components-parameters",
+            )
+        for param_node in iter(param_nodes):
+            path_nodes = set(shacl_graph.objects(param_node, SH_path))
+            if len(path_nodes) < 1:
+                # TODO:coverage: we don't have any tests for invalid constraints
+                raise ConstraintLoadError(
+                    "A sh:ConstraintComponent parameter value must have at least one value for sh:path",
+                    "https://www.w3.org/TR/shacl/#constraint-components-parameters",
                 )
-            except ValidationFailure as e:
-                raise e
-            if not self.shape.is_property_shape:
-                result_val = f
+            elif len(path_nodes) > 1:
+                # TODO:coverage: we don't have any tests for invalid constraints
+                raise ConstraintLoadError(
+                    "A sh:ConstraintComponent parameter value must have at most one value for sh:path",
+                    "https://www.w3.org/TR/shacl/#constraint-components-parameters",
+                )
+            path = next(iter(path_nodes))
+            parameter = SHACLParameter(shacl_graph, param_node, path=path, logger=None)  # pass in logger?
+            if parameter.optional:
+                optional_params.append(parameter)
             else:
-                result_val = None
-            for v in violations:
-                non_conformant = True
-                if isinstance(v, bool) and v is True:
-                    # TODO:coverage: No test for when violation is `True`
-                    rept = self.make_v_result(target_graph, f, value_node=result_val, **rept_kwargs)
-                elif isinstance(v, tuple):
-                    t, p, v = v
-                    if v is None:
-                        v = result_val
-                    rept = self.make_v_result(target_graph, t or f, value_node=v, result_path=p, **rept_kwargs)
-                else:
-                    rept = self.make_v_result(target_graph, f, value_node=v, **rept_kwargs)
-                reports.append(rept)
-        return (not non_conformant), reports
+                mandatory_params.append(parameter)
+        if len(mandatory_params) < 1:
+            # TODO:coverage: we don't have any tests for invalid constraint components
+            raise ConstraintLoadError(
+                "A sh:ConstraintComponent must have at least one non-optional parameter.",
+                "https://www.w3.org/TR/shacl/#constraint-components-parameters",
+            )
+        self.append(mandatory_params + optional_params)
+
+        validator_node_set = set(shacl_graph.graph.objects(node, SH_validator))
+        node_val_node_set = set(shacl_graph.graph.objects(node, SH_nodeValidator))
+        prop_val_node_set = set(shacl_graph.graph.objects(node, SH_propertyValidator))
+        validator_node_set = validator_node_set.difference(node_val_node_set)
+        validator_node_set = validator_node_set.difference(prop_val_node_set)
+        self.append(validator_node_set)
+        self.append(node_val_node_set)
+        self.append(prop_val_node_set)
+        is_sparql_constraint_component = False
+        for s in (validator_node_set, node_val_node_set, prop_val_node_set):
+            for v in s:
+                v_types = set(shacl_graph.graph.objects(v, RDF_type))
+                if SH_SPARQLAskValidator in v_types or SH_SPARQLSelectValidator in v_types:
+                    is_sparql_constraint_component = True
+                    break
+                v_props = set(p[0] for p in shacl_graph.graph.predicate_objects(v))
+                if SH_ask in v_props or SH_select in v_props:
+                    is_sparql_constraint_component = True
+                    break
+                if is_sparql_constraint_component:
+                    raise ConstraintLoadError(
+                        "Found a mix of SPARQL-based validators and non-SPARQL validators on a SPARQLConstraintComponent.",
+                        'https://www.w3.org/TR/shacl/#constraint-components-validators',
+                    )
+        if is_sparql_constraint_component:
+            return SPARQLConstraintComponent(*self)
+        else:
+            return CustomConstraintComponent(*self)
 
 
-class SPARQLConstraintComponent(object):
+class CustomConstraintComponent(object):
+    __slots__: Tuple = ('sg', 'node', 'parameters', 'validators', 'node_validators', 'property_validators')
+
+    if typing.TYPE_CHECKING:
+        sg: ShapesGraph
+        node: Any
+        parameters: List[SHACLParameter]
+        validators: Set
+        node_validators: Set
+        property_validators: Set
+
+    def __new__(cls, shacl_graph: 'ShapesGraph', node, parameters, validators, node_validators, property_validators):
+        self = super(CustomConstraintComponent, cls).__new__(cls)
+        self.sg = shacl_graph
+        self.node = node
+        self.parameters = parameters
+        self.validators = validators
+        self.node_validators = node_validators
+        self.property_validators = property_validators
+        return self
+
+    def make_validator_for_shape(self, shape: 'Shape'):
+        raise NotImplementedError()
+
+
+class SPARQLConstraintComponent(CustomConstraintComponent):
     """
     SPARQL-based constraints provide a lot of flexibility but may be hard to understand for some people or lead to repetition. This section introduces SPARQL-based constraint components as a way to abstract the complexity of SPARQL and to declare high-level reusable components similar to the Core constraint components. Such constraint components can be declared using the SHACL RDF vocabulary and thus shared and reused.
     Link:
     https://www.w3.org/TR/shacl/#sparql-constraint-components
     """
 
-    def __init__(self, shacl_graph, node, mandatory_parameters, optional_parameters):
-        self.sg = shacl_graph
-        self.node = node
-        self.mandatory_parameters = mandatory_parameters
-        self.optional_parameters = optional_parameters
-        validator_node_list = set(self.sg.graph.objects(node, SH_validator))
-        node_val_node_list = set(self.sg.graph.objects(node, SH_nodeValidator))
-        prop_val_node_list = set(self.sg.graph.objects(node, SH_propertyValidator))
-        self.validator_nodes = validator_node_list
-        self.node_validator_nodes = node_val_node_list
-        self.prop_validator_nodes = prop_val_node_list
-        val_count = len(self.validator_nodes)
-        node_val_count = len(self.node_validator_nodes)
-        prop_val_count = len(self.prop_validator_nodes)
-        if (val_count + node_val_count + prop_val_count) < 1:
-            # TODO:coverage: No test for this case, do we need to test this?
-            raise ConstraintLoadError(
-                "ConstraintComponent must have at least one sh:validator, "
-                "sh:nodeValidator, or sh:propertyValidator predicates.",
-                "https://www.w3.org/TR/shacl/#ConstraintComponent",
-            )
+    __slots__: Tuple = tuple()
 
-    @classmethod
-    def parameter_name(cls, parameter):
-        path = str(parameter.path())
-        hash_index = path.find('#')
-        if hash_index > 0:
-            ending = path[hash_index + 1 :]
-            return ending
-        right_slash_index = path.rfind('/')
-        if right_slash_index > 0:
-            # TODO:coverage: No test for this case where path has a right slash
-            ending = path[right_slash_index + 1 :]
-            return ending
-        raise ReportableRuntimeError("Cannot get a local name for {}".format(path))
+    def __new__(cls, shacl_graph, node, parameters, validators, node_validators, property_validators):
+        return super(SPARQLConstraintComponent, cls).__new__(
+            cls, shacl_graph, node, parameters, validators, node_validators, property_validators
+        )
 
-    def make_validator_for_shape(self, shape):
+    def make_validator_for_shape(self, shape: 'Shape'):
         """
         :param shape:
-        :type shape: pyshacl.shape.Shape
+        :type shape: Shape
         :return:
         """
-        val_count = len(self.validator_nodes)
-        node_val_count = len(self.node_validator_nodes)
-        prop_val_count = len(self.prop_validator_nodes)
+        val_count = len(self.validators)
+        node_val_count = len(self.node_validators)
+        prop_val_count = len(self.property_validators)
         must_be_select_val = False
         must_be_ask_val = False
         if shape.is_property_shape and prop_val_count > 0:
-            validator_node = next(iter(self.prop_validator_nodes))
+            validator_node = next(iter(self.property_validators))
             must_be_select_val = True
         elif (not shape.is_property_shape) and node_val_count > 0:
-            validator_node = next(iter(self.node_validator_nodes))
+            validator_node = next(iter(self.node_validators))
             must_be_select_val = True
         elif val_count > 0:
-            validator_node = next(iter(self.validator_nodes))
+            validator_node = next(iter(self.validators))
             must_be_ask_val = True
         else:
             raise ConstraintLoadError(
