@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
 #
 import logging
+import sys
 
 from functools import wraps
+from os import path
 from sys import stderr
 from typing import Dict, List, Optional, Set, Tuple, Union
 
@@ -11,7 +13,8 @@ import rdflib
 
 from rdflib.util import from_n3
 
-from pyshacl.pytypes import GraphLike
+from .pytypes import GraphLike
+from .target import apply_target_types, gather_target_types
 
 
 if owlrl.json_ld_available:
@@ -19,7 +22,7 @@ if owlrl.json_ld_available:
 
 from rdflib import BNode, Literal, URIRef
 
-from pyshacl.consts import (
+from .consts import (
     RDF_object,
     RDF_predicate,
     RDF_subject,
@@ -30,11 +33,11 @@ from pyshacl.consts import (
     SH_resultMessage,
     SH_ValidationReport,
 )
-from pyshacl.errors import ReportableRuntimeError, ValidationFailure
-from pyshacl.functions import apply_functions, gather_functions, unapply_functions
-from pyshacl.inference import CustomRDFSOWLRLSemantics, CustomRDFSSemantics
-from pyshacl.monkey import apply_patches, rdflib_bool_patch, rdflib_bool_unpatch
-from pyshacl.rdfutil import (
+from .errors import ReportableRuntimeError, ValidationFailure
+from .functions import apply_functions, gather_functions, unapply_functions
+from .inference import CustomRDFSOWLRLSemantics, CustomRDFSSemantics
+from .monkey import apply_patches, rdflib_bool_patch, rdflib_bool_unpatch
+from .rdfutil import (
     clone_blank_node,
     clone_graph,
     compare_blank_node,
@@ -44,8 +47,9 @@ from pyshacl.rdfutil import (
     mix_graphs,
     order_graph_literal,
 )
-from pyshacl.rules import apply_rules, gather_rules
-from pyshacl.shapes_graph import ShapesGraph
+from .rdfutil.load import add_baked_in
+from .rules import apply_rules, gather_rules
+from .shapes_graph import ShapesGraph
 
 
 log_handler = logging.StreamHandler(stderr)
@@ -202,9 +206,11 @@ class Validator(object):
 
         shapes = self.shacl_graph.shapes  # This property getter triggers shapes harvest.
         if self.options['advanced']:
+            target_types = gather_target_types(self.shacl_graph)
             advanced = {'functions': gather_functions(self.shacl_graph), 'rules': gather_rules(self.shacl_graph)}
             for s in shapes:
                 s.set_advanced(True)
+            apply_target_types(target_types)
         else:
             advanced = {}
         if isinstance(the_target_graph, (rdflib.Dataset, rdflib.ConjunctiveGraph)):
@@ -218,6 +224,7 @@ class Validator(object):
             named_graphs = [the_target_graph]
         reports = []
         non_conformant = False
+
         for g in named_graphs:
             if advanced:
                 apply_functions(advanced['functions'], g)
@@ -232,6 +239,22 @@ class Validator(object):
         return (not non_conformant), v_report, v_text
 
 
+def assign_baked_in():
+    if getattr(sys, 'frozen', False):
+        # runs in a pyinstaller bundle
+        HERE = sys._MEIPASS
+    else:
+        HERE = path.dirname(__file__)
+    shacl_file = path.join(HERE, "assets", "shacl.pickle")
+    add_baked_in("http://www.w3.org/ns/shacl", shacl_file)
+    add_baked_in("https://www.w3.org/ns/shacl", shacl_file)
+    add_baked_in("http://www.w3.org/ns/shacl.ttl", shacl_file)
+    shacl_shacl_file = path.join(HERE, "assets", "shacl-shacl.pickle")
+    add_baked_in("http://www.w3.org/ns/shacl-shacl", shacl_shacl_file)
+    add_baked_in("https://www.w3.org/ns/shacl-shacl", shacl_shacl_file)
+    add_baked_in("http://www.w3.org/ns/shacl-shacl.ttl", shacl_shacl_file)
+
+
 def with_metashacl_shacl_graph_cache(f):
     # noinspection PyPep8Naming
     EMPTY = object()
@@ -242,21 +265,17 @@ def with_metashacl_shacl_graph_cache(f):
         assert graph_cache is not None
         if graph_cache is EMPTY:
             import pickle
-            import sys
-
-            from os import path
 
             if getattr(sys, 'frozen', False):
                 # runs in a pyinstaller bundle
                 here_dir = sys._MEIPASS
-                pickle_file = path.join(here_dir, "shacl-shacl.pickle")
             else:
                 here_dir = path.dirname(__file__)
-                pickle_file = path.join(here_dir, "shacl-shacl.pickle")
+            pickle_file = path.join(here_dir, "assets", "shacl-shacl.pickle")
             with open(pickle_file, 'rb') as shacl_pickle:
                 u = pickle.Unpickler(shacl_pickle, fix_imports=False)
-                shacl_shacl_store = u.load()
-            shacl_shacl_graph = rdflib.Graph(store=shacl_shacl_store, identifier="http://www.w3.org/ns/shacl-shacl")
+                shacl_shacl_store, identifier = u.load()
+            shacl_shacl_graph = rdflib.Graph(store=shacl_shacl_store, identifier=identifier)
             setattr(wrapped, "graph_cache", shacl_shacl_graph)
         return f(*args, **kwargs)
 
@@ -306,6 +325,7 @@ def validate(
         log_handler.setLevel(logging.DEBUG)
         log.setLevel(logging.DEBUG)
     apply_patches()
+    assign_baked_in()
     do_check_dash_result = kwargs.pop('check_dash_result', False)  # type: bool
     do_check_sht_result = kwargs.pop('check_sht_result', False)  # type: bool
     if kwargs.get('meta_shacl', False):
